
#include <stdint.h>

#include "base.h"
#include "ipc.h"
#include "gpio.h"
#include "drivers.h"
#include "interrupts.h"
#include "scheduler.h"
#include "uart.h"
#include "time.h"

#define GPIO_BASE 	( PERIPHERAL_BASE + 0x00200000 )

static gpio_t* Gpio = (gpio_t*)GPIO_BASE;

gpio_t* get_gpio(void)
{
    return Gpio;
}

void gpio_init(void){
    irq_controller_get()->Enable_IRQs_2 |= GPIO_IRQ;
}

void gpio_jtag_enable(void){
    // set gpio4 to alt 5
    gpio_t *gpio = get_gpio();
    gpio->GPFSEL0 &= ~(7 << 12);    // zero out gpio4
    gpio->GPFSEL0 |= 2<< 12;    //gpio4 to alt5 = ARM_TDI
    
    // set other gpios
    gpio->GPFSEL2 &= ~(     
                        (7<<6)      //zero gpio22
                       |(7<<12)     // gpio24
                       |(7<<15)     // gpio25
                       |(7<<21));   // gpio27
    
    gpio->GPFSEL2 |=    (3 << 6)    // alt4 ARM TRST
                       |(3 << 12)   // alt4 ARM_TD0
                       |(3 << 15)   // alt4 ARM_TCK
                       |(3 << 21);   // alt4 ARM_TMS

    return;
}

void gpio_emmc_enable(void){
    gpio_t *gpio = get_gpio();
    // set as input
    gpio->GPFSEL4 &= ~(7 << 23);
    
}

void gpio_ack(void){
    get_gpio()->GPEDS0 = 0xffffffff;
    get_gpio()->GPEDS1 = 0xffffffff;
}

#define GPIO_IRQS       (0xf << 17)
scheduling_type_t gpio_handler(void){
    // send msg with interrupt regs
    gpio_ack();
    ipc_kernel_send(NULL, 0, driver_irq_get(DRIVER_GPIO));
    return RESCHEDULE;
}
