
#ifndef MAILBOX_H
#define MAILBOX_H
#include <stdint.h>

void mailbox_write(volatile uint32_t *addr, uint8_t chan);

uint32_t mailbox_read(uint8_t chan);

uint32_t mailbox_write_read(volatile uint32_t *addr, uint8_t chan);

//  Clock IDs
enum{
    MAILBOX_CLOCK_EMMC          = 1,
    MAILBOX_CLOCK_UART          = 2,
    MAILBOX_CLOCK_ARM           = 3,
    MAILBOX_CLOCK_CORE          = 4,
    MAILBOX_CLOCK_V3D           = 5,
    MAILBOX_CLOCK_H264          = 6,
    MAILBOX_CLOCK_ISP           = 7,
    MAILBOX_CLOCK_SDRAM         = 8,
    MAILBOX_CLOCK_PIXEL         = 9,
    MAILBOX_CLOCK_PWM           = 10,
};

//  Unique power IDs
enum{
    MAILBOX_SD_CARD             = 0,
    MAILBOX_UART0               = 1,
    MAILBOX_UART1               = 2,
    MAILBOX_USB                 = 3,
    MAILBOX_I2C0                = 4,
    MAILBOX_I2C1                = 5,
    MAILBOX_I2C2                = 6,
    MAILBOX_SPI                 = 7,
    MAILBOX_CCP2TX              = 8,
};


// Mailbox channels
enum{
    MAILBOX_POWER_MANAGEMENT    = 0,
    MAILBOX_FRAMEBUFFER         = 1,
    MAILBOX_VITUAL_UART         = 2,
    MAILBOX_VCHIQ               = 3,
    MAILBOX_LEDS                = 4,
    MAILBOX_BUTTONS             = 5,
    MAILBOX_TOUCHSCREEN         = 6,
    MAILBOX_ARM_TO_VC           = 8,
    MAILBOX_VC_TO_ARM           = 9,
};

enum {  
    MAILBOX_REQUEST = 0,
    MAILBOX_REQUEST_SUCCESSFUL 	= 0x80000000,
    MAILBOX_REQUEST_FAILED 		= 0x80000001
};
/* Mailbox property tags */
enum {
	MAILBOX_PROPERTY_END                = 0x00000000,
	MAILBOX_GET_FIRMWARE_REVISION       = 0x00000001,
	MAILBOX_GET_BOARD_MODEL             = 0x00010001,
	MAILBOX_GET_BOARD_REVISION	        = 0x00010002,
	MAILBOX_GET_BOARD_MAC_ADDRESS	    = 0x00010003,
	MAILBOX_GET_BOARD_SERIAL		    = 0x00010004,
	MAILBOX_GET_ARM_MEMORY		        = 0x00010005,
	MAILBOX_GET_VC_MEMORY		        = 0x00010006,
	MAILBOX_GET_CLOCKS		            = 0x00010007,
	MAILBOX_GET_COMMAND_LINE            = 0x00050001,
	MAILBOX_GET_DMA_CHANNELS            = 0x00060001,
	MAILBOX_GET_POWER_STATE             = 0x00020001,
	MAILBOX_GET_TIMING		            = 0x00020002,
	MAILBOX_SET_POWER_STATE             = 0x00028001,
	MAILBOX_GET_CLOCK_STATE             = 0x00030001,
	MAILBOX_SET_CLOCK_STATE             = 0x00038001,
	MAILBOX_GET_CLOCK_RATE              = 0x00030002,
	MAILBOX_SET_CLOCK_RATE              = 0x00038002,
	MAILBOX_GET_VOLTAGE                 = 0x00030003,
	MAILBOX_SET_VOLTAGE                 = 0x00038003,
	MAILBOX_GET_MAX_CLOCK               = 0x00030004,
	MAILBOX_GET_MAX_VOLTAGE             = 0x00030005,
	MAILBOX_GET_TEMPERATURE             = 0x00030006,
	MAILBOX_GET_MIN_CLOCK               = 0x00030007,
	MAILBOX_GET_MIN_VOLTAGE             = 0x00030008,
	MAILBOX_GET_TURBO                   = 0x00030009,
	MAILBOX_GET_MAX_TEMPERATURE         = 0x0003000a,
	MAILBOX_GET_STC                     = 0x0003000b,
	MAILBOX_SET_TURBO                   = 0x00038009,
	MAILBOX_SET_ALLOCATE_MEM            = 0x0003000c,
	MAILBOX_SET_LOCK_MEM                = 0x0003000d,
	MAILBOX_SET_UNLOCK_MEM              = 0x0003000e,
	MAILBOX_SET_RELEASE_MEM             = 0x0003000f,
	MAILBOX_SET_EXECUTE_CODE            = 0x00030010,
	MAILBOX_SET_EXECUTE_QPU             = 0x00030011,
	MAILBOX_SET_ENABLE_QPU              = 0x00030012,
	MAILBOX_GET_RESOURCE_HANDLE         = 0x00030014,
	MAILBOX_GET_EDID_BLOCK              = 0x00030020,
	MAILBOX_GET_CUSTOMER_OTP            = 0x00030021,
	MAILBOX_SET_CUSTOMER_OTP            = 0x00038021,
	MAILBOX_SET_ALLOCATE_BUFFER         = 0x00040001,
	MAILBOX_SET_RELEASE_BUFFER          = 0x00048001,
	MAILBOX_SET_BLANK_SCREEN            = 0x00040002,
	MAILBOX_TST_BLANK_SCREEN            = 0x00044002,
	MAILBOX_GET_PHYSICAL_WIDTH_HEIGHT   = 0x00040003,
	MAILBOX_TST_PHYSICAL_WIDTH_HEIGHT   = 0x00044003,
	MAILBOX_SET_PHYSICAL_WIDTH_HEIGHT   = 0x00048003,
	MAILBOX_GET_VIRTUAL_WIDTH_HEIGHT    = 0x00040004,
	MAILBOX_TST_VIRTUAL_WIDTH_HEIGHT    = 0x00044004,
	MAILBOX_SET_VIRTUAL_WIDTH_HEIGHT    = 0x00048004,
	MAILBOX_GET_DEPTH                   = 0x00040005,
	MAILBOX_TST_DEPTH                   = 0x00044005,
	MAILBOX_SET_DEPTH                   = 0x00048005,
	MAILBOX_GET_PIXEL_ORDER             = 0x00040006,
	MAILBOX_TST_PIXEL_ORDER             = 0x00044006,
	MAILBOX_SET_PIXEL_ORDER             = 0x00048006,
	MAILBOX_GET_ALPHA_MODE              = 0x00040007,
	MAILBOX_TST_ALPHA_MODE              = 0x00044007,
	MAILBOX_SET_ALPHA_MODE              = 0x00048007,
	MAILBOX_GET_PITCH                   = 0x00040008,
	MAILBOX_TST_PITCH                   = 0x00044008,
	MAILBOX_SET_PITCH                   = 0x00048008,
	MAILBOX_GET_VIRTUAL_OFFSET          = 0x00040009,
	MAILBOX_TST_VIRTUAL_OFFSET          = 0x00044009,
	MAILBOX_SET_VIRTUAL_OFFSET          = 0x00048009,
	MAILBOX_GET_OVERSCAN                = 0x0004000a,
	MAILBOX_TST_OVERSCAN                = 0x0004400a,
	MAILBOX_SET_OVERSCAN                = 0x0004800a,
	MAILBOX_GET_PALETTE                 = 0x0004000b,
	MAILBOX_TST_PALETTE                 = 0x0004400b,
	MAILBOX_SET_PALETTE                 = 0x0004800b,
	MAILBOX_GET_LAYER                   = 0x0004000c,
	MAILBOX_TST_LAYER                   = 0x0004400c,
	MAILBOX_SET_LAYER                   = 0x0004800c,
	MAILBOX_GET_TRANSFORM               = 0x0004000d,
	MAILBOX_TST_TRANSFORM               = 0x0004400d,
	MAILBOX_SET_TRANSFORM               = 0x0004800d,
	MAILBOX_TST_VSYNC                   = 0x0004400e,
	MAILBOX_SET_VSYNC                   = 0x0004800e,
	MAILBOX_SET_CURSOR_INFO             = 0x00008010,
	MAILBOX_SET_CURSOR_STATE            = 0x00008011,
};
#endif
